@@ -117,23 +117,6 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
 
     @Override
     public void handleBookiesThatLeft(Set<BookieId> leftBookies) {
-        // case 1: (In some situation, eg, Broker and bookie restart concurrently.)
-        //1. Bookie X join cluster for the first time, encounters a region exception, and `address2Region` record X's
-        // region as default-region.
-        //2. Bookie X left cluster and is removed from knownBookies, but address2Region retains the information of
-        // bookie X.
-        //3. update Bookie X's rack info, and calling `onBookieRackChange` will only update address2Region for
-        // addresses present in knownBookies; therefore, bookie X's region info is not updated.
-        //4. Bookie X join cluster again, since address2Region contains the previous default-region information,
-        // getRegion will directly use cached data, resulting of an incorrect region.
-
-        // The bookie region is initialized to "default-region" in address2Region.
-        // We should ensure that when a bookie leaves the cluster,
-        // we also clean up the corresponding region information for that bookie in address2Region,
-        // so that it can update the correct region for the bookie during onBookieRackChange and
-        // handleBookiesThatJoined.
-        // to avoid traffic skew in ensemble selection.
-        leftBookies.forEach(address2Region::remove);
         super.handleBookiesThatLeft(leftBookies);
 
         for (TopologyAwareEnsemblePlacementPolicy policy: perRegionPlacement.values()) {
@@ -157,7 +140,8 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
                         .initialize(dnsResolver, timer, this.reorderReadsRandom, this.stabilizePeriodSeconds,
                                 this.reorderThresholdPendingRequests, this.isWeighted, this.maxWeightMultiple,
                                 this.minNumRacksPerWriteQuorum, this.enforceMinNumRacksPerWriteQuorum,
-                                this.ignoreLocalNodeInPlacementPolicy, statsLogger, bookieAddressResolver)
+                                this.ignoreLocalNodeInPlacementPolicy,
+                                this.useHostnameResolveLocalNodePlacementPolicy, statsLogger, bookieAddressResolver)
                         .withDefaultRack(NetworkTopology.DEFAULT_REGION_AND_RACK));
             }
 
@@ -218,7 +202,8 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
                                                 this.stabilizePeriodSeconds, this.reorderThresholdPendingRequests,
                                                 this.isWeighted, this.maxWeightMultiple,
                                                 this.minNumRacksPerWriteQuorum, this.enforceMinNumRacksPerWriteQuorum,
-                                                this.ignoreLocalNodeInPlacementPolicy, statsLogger,
+                                                this.ignoreLocalNodeInPlacementPolicy,
+                                                this.useHostnameResolveLocalNodePlacementPolicy, statsLogger,
                                                 bookieAddressResolver)
                                         .withDefaultRack(NetworkTopology.DEFAULT_REGION_AND_RACK);
                                 perRegionPlacement.put(newRegion, newRegionPlacement);
@@ -259,7 +244,8 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
                         .initialize(dnsResolver, timer, this.reorderReadsRandom, this.stabilizePeriodSeconds,
                                 this.reorderThresholdPendingRequests, this.isWeighted, this.maxWeightMultiple,
                                 this.minNumRacksPerWriteQuorum, this.enforceMinNumRacksPerWriteQuorum,
-                                this.ignoreLocalNodeInPlacementPolicy, statsLogger, bookieAddressResolver)
+                                this.ignoreLocalNodeInPlacementPolicy, this.useHostnameResolveLocalNodePlacementPolicy,
+                                statsLogger, bookieAddressResolver)
                         .withDefaultRack(NetworkTopology.DEFAULT_REGION_AND_RACK));
             }
             minRegionsForDurability = conf.getInt(REPP_MINIMUM_REGIONS_FOR_DURABILITY,
@@ -406,9 +392,20 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
                 remainingEnsembleBeforeIteration = remainingEnsemble;
                 int regionsToAllocate = numRemainingRegions;
                 int startRegionIndex = lastRegionIndex % numRegionsAvailable;
+                int localRegionIndex = -1;
+                if (myRegion != null && !UNKNOWN_REGION.equals(myRegion)) {
+                    localRegionIndex = availableRegions.indexOf(myRegion);
+                }
+                String region = myRegion;
                 for (int i = 0; i < numRegionsAvailable; ++i) {
-                    String region = availableRegions.get(startRegionIndex % numRegionsAvailable);
-                    startRegionIndex++;
+                    // select the local region first, and for the rest region select, use round-robin selection.
+                    if (i > 0 || localRegionIndex == -1) {
+                        if (startRegionIndex % numRegionsAvailable == localRegionIndex) {
+                            startRegionIndex++;
+                        }
+                        region = availableRegions.get(startRegionIndex % numRegionsAvailable);
+                        startRegionIndex++;
+                    }
                     final Pair<Integer, Integer> currentAllocation = regionsWiseAllocation.get(region);
                     TopologyAwareEnsemblePlacementPolicy policyWithinRegion = perRegionPlacement.get(region);
                     if (!regionsReachedMaxAllocation.contains(region)) {
@@ -666,12 +663,4 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
          */
         return PlacementPolicyAdherence.MEETS_STRICT;
     }
-
-    @Override
-    public void updateBookieInfo(Map<BookieId, BookieInfoReader.BookieInfo> bookieInfoMap) {
-        super.updateBookieInfo(bookieInfoMap);
-        for (TopologyAwareEnsemblePlacementPolicy policy : perRegionPlacement.values()) {
-            policy.updateBookieInfo(bookieInfoMap);
-        }
-    }
 }
