@@ -53,8 +53,18 @@ public class BookieStateReadOnlyService implements HttpEndpointService {
         if (HttpServer.Method.PUT.equals(request.getMethod())) {
             ReadOnlyState inState = JsonUtil.fromJson(request.getBody(), ReadOnlyState.class);
             if (stateManager.isReadOnly() && !inState.isReadOnly()) {
+                if (stateManager.isForceReadOnly()) {
+                    response.setCode(HttpServer.StatusCode.BAD_REQUEST);
+                    response.setBody("Bookie is in forceReadOnly mode, cannot transit to writable mode");
+                    return response;
+                }
                 stateManager.transitionToWritableMode().get();
             } else if (!stateManager.isReadOnly() && inState.isReadOnly()) {
+                if (!stateManager.isReadOnlyModeEnabled()) {
+                    response.setCode(HttpServer.StatusCode.BAD_REQUEST);
+                    response.setBody("Bookie is disabled ReadOnly mode, cannot transit to readOnly mode");
+                    return response;
+                }
                 stateManager.transitionToReadOnlyMode().get();
             }
         } else if (!HttpServer.Method.GET.equals(request.getMethod())) {
